/* Copyright (C) 2000 Stefan Karrmann <S.Karrmann@gmx.net>
 * All Rights reserved.
 */

/* $Id: random.c,v 1.2 2000/02/19 22:29:07 sk Exp $
 * $Log: random.c,v $
 * Revision 1.2  2000/02/19 22:29:07  sk
 * bersetzbar - mit Laufzeitfehlern
 *
 * Revision 1.1  2000/02/12 22:20:18  sk
 * Switched to C
 *
 * Revision 1.1  2000/02/11 22:26:22  sk
 * Kommentarfehler bearbeitet, miese Zufallszahlenschnittstelle
 *
 * Revision 1.1  2000/02/11 22:10:17  sk
 * solve und fast generate erstellt
 */

#include "config.h"
#include "sharesecret.h"

#define PRIME_PROBABILITY (30)

inline static void
gen_rand (mpz_t r, ulong size)
{
  ulong limbSize, allocedSize;

  /* precondition */
  I(size + 7 > 0);

  limbSize = ((size + 7) / 8 + mp_bits_per_limb - 1) / mp_bits_per_limb;
  allocedSize = limbSize * mp_bits_per_limb * 8;
  mpz_random (r, limbSize);
  while (size < allocedSize)
    {
      size = size + 1;
      mpz_clrbit (r, size);
    }
}

/* *init* and set r */
/* void gen_random (mpz_t r, RNG g, ulong size); */
void
gen_random (mpz_t r, ulong size)
{
  mpz_init (r);
  gen_rand (r, size);
}

/* *init and set prime */

/* void generate_prime (mpz_t prime, RNG g, ulong size); */
void
generate_prime (mpz_t prime, ulong size)
{
  mpz_init (prime);
  do
    {
      gen_rand (prime, size);
      mpz_setbit (prime, 0);
      mpz_setbit (prime, size);
    }
  while (0 == mpz_probab_prime_p (prime, PRIME_PROBABILITY));
  /* error probability 0.25^30 */
}
