/* Copyright (C) 2000 Stefan Karrmann <S.Karrmann@gmx.net>
 * All Rights reserved.
 */

/* $Id: solve.c,v 1.4 2000/02/23 21:19:58 sk Exp $
 * $Log: solve.c,v $
 * Revision 1.4  2000/02/23 21:19:58  sk
 * First working Version
 *
 * Revision 1.3  2000/02/19 22:29:07  sk
 * bersetzbar - mit Laufzeitfehlern
 *
 * Revision 1.2  2000/02/13 23:12:32  sk
 * worked on commandline options
 *
 * Revision 1.1  2000/02/12 22:20:18  sk
 * Switched to C
 *
 * Revision 1.2  2000/02/11 22:26:22  sk
 * Kommentarfehler bearbeitet, miese Zufallszahlenschnittstelle
 *
 * Revision 1.1  2000/02/11 22:10:17  sk
 * solve und fast generate erstellt
 */

#include "sharesecret.h"

/* Using the Aitken Neville Algorithmus to calculate the secret
 *  polynomial at x = 0:
 *
 * init: p_j..j(x) = y[j]
 * recusion equation:
 * p_j..k(x) = 1/(x_k-x_j)*((x_k-x)*p_j..k-1(x)-(x_j-x)*p_j+1..k(x))
 */

/* initializes result, callee must (mpz_)clear it.
 * changes y !
 */

void
solve (mpz_t result, mpz_t modulus, size_t size,
       mpz_t * x, mpz_t * y)
{
  ulong m;			/* degree of polynomial */
  ulong i, j;
  mpz_t tmpz;

  /* init */
  mpz_init (tmpz);
  m = size - 1;			/* degree of polynomial */

  /* solve */
  for (i = 1;  i < m + 1; i++)
    {
      /* step */
      for (j = 0; j < m + 1 - i; j++)
	{
	  /* calculate: y[j] = ((x[j+i] * y[j] - x[j] * y[j+1])
	                       / (x[j+i] - x[j]));
	  */
	  mpz_mul (y[j], x[j+i], y[j]);
	  mpz_mod (y[j], y[j], modulus);
	  mpz_mul (tmpz, x[j], y[j+1]);
	  mpz_mod (tmpz, tmpz, modulus);
	  mpz_sub (y[j], y[j], tmpz);
	  if ( 0 > mpz_sgn (y[j])) /* I'm very cautious */
	    mpz_add (y[j], y[j], modulus);

	  /* invariant */
	  I(0 <= mpz_sgn (y[j]) && 0 > mpz_cmp (y[j], modulus));

	  /* multi-reuse tmpz, using mpz is like assembler */
	  mpz_sub (tmpz, x[j+i], x[j]);
	  if ( 0 > mpz_sgn (tmpz)) /* I'm very cautious */
	    mpz_add (tmpz, tmpz, modulus);

	  /* invariant, don't divide by zero */
	  I(0 < mpz_sgn (tmpz) && 0 > mpz_cmp (tmpz, modulus));

	  mpz_invert (tmpz, tmpz, modulus);
	  mpz_mul (y[j], y[j], tmpz);
	  mpz_mod (y[j], y[j], modulus);
	}
    }

  /*  resulting */
  mpz_init_set (result, y[0]);

  /* cleanup */
  mpz_clear (tmpz);
}

/* initializes result, callee must (mpz_)clear it. */
void
xor_solve (mpz_t result, size_t size, mpz_t * x)
{
  size_t i;

  /* init */
  mpz_init (result);
  
  /* solve*/
  for (i = 0; i < size; i++)
    mpz_xor (result, result, x[i]);
}
